@@ -26,6 +26,7 @@ for more information.
 import errno
 import inspect
 import socket as sock
+import sys
 import time as time
 
 import six
@@ -41,6 +42,14 @@ from .sptype import JsonObject, spType, either, const, maybe, longType
 from .sputils import msec, sec, pathPollWait
 from .spdoc import ApiDoc, ApiCallDoc
 
+if sys.version_info[0] < 3:
+    import urllib as uquote
+else:
+    import urllib.parse as uquote
+
+
+VERSION = '5.5.0'
+
 
 SP_DEV_PATH = '/dev/storpool/'
 SP_API_PREFIX = '/ctrl/1.0'
@@ -60,6 +69,12 @@ class _API_ARG(object):
         self._name = name
         self._type = spType(validate)
 
+    def defstr(self):
+        if self._type.spDoc.name == "Optional":
+            return "{name}=None".format(name=self._name)
+        else:
+            return self._name
+
 
 DiskId = _API_ARG('diskId', sp.DiskId)
 ServerId = _API_ARG('serverId', sp.ServerId)
@@ -103,7 +118,7 @@ class _API_METHOD(object):
             args.append(_API_ARG('json', json))
 
         ftext = 'def func(self, {args}clusterName=None):\n'.format(
-            args=''.join(arg._name + ", " for arg in args))
+            args=''.join(arg.defstr() + ", " for arg in args))
         for arg in args:
             ftext += '    {arg} = _validate_{arg}({arg})\n'.format(arg=arg._name)
 
@@ -147,9 +162,8 @@ class _API_METHOD(object):
 
 
 def GET(query, *args, **kwargs):
-    assert 'json' not in kwargs, 'GET requests currently do not accept JSON objects'
     assert 'returns' in kwargs, 'GET requests must specify a return type'
-    return _API_METHOD('GET', kwargs.get('multiCluster', False), query, args, None, kwargs['returns'])
+    return _API_METHOD('GET', kwargs.get('multiCluster', False), query, args, kwargs.get('json', None), kwargs['returns'])
 
 
 def POST(query, *args, **kwargs):
@@ -186,6 +200,19 @@ class ApiOkVolumesGroupBackup(ApiOk):
     '''
 
 
+@JsonObject(
+    autoName=sp.maybe(sp.SnapshotName),
+    snapshotGlobalId=sp.maybe(sp.GlobalVolumeId),
+    snapshotVisibleVolumeId=sp.maybe(sp.longType)
+)
+class ApiOkSnapshotCreate(ApiOk):
+    '''
+    autoName: The name of the transient snapshot used during the creation of the volume.
+    snapshotGlobalId: The global snapshot identifier.
+    snapshotVisibleVolumeId: The ID by which the volume/snapshot was created.
+    '''
+
+
 class ApiError(Exception):
     def __init__(self, status, json):
         super(ApiError, self).__init__()
@@ -287,6 +314,9 @@ class Api(object):
             try:
                 conn = http.HTTPConnection(self._host, self._port, timeout=self._timeout, **self._source)
                 path = _format_path(query, multiCluster and self._multiCluster, clusterName=clusterName)
+                if method == "GET" and json:
+                    path += "?json=" + uquote.quote(json, safe='')
+                    json = None
                 conn.request(method, path, json, self._authHeader)
                 response = conn.getresponse()
                 status, jres = response.status, js.load(response)
@@ -581,6 +611,10 @@ Api.volumeAcquire = POST('VolumeAcquire/{volumeName}', VolumeName, json=sp.Volum
    """
    """)
 
+Api.volumeFromRemote = POST('VolumeFromRemote', json=sp.VolumeFromRemoteDesc).doc("Create a volume from a snapshot from a remote location",
+  """
+  """)
+
 Api.spDocSection("Snapshots",
     """
     Snapshots in their essence are very similar to volumes in the sense
@@ -588,7 +622,7 @@ Api.spDocSection("Snapshots",
     snapshots (all except write-related operations). They can not be
     modified and play an essential role in copy-on-write scenarios.
     """)
-Api.snapshotsList = GET('SnapshotsList', returns=[sp.SnapshotSummary]).doc("List all snapshots",
+Api.snapshotsList = GET('SnapshotsList', returns=[sp.SnapshotSummary], multiCluster=True).doc("List all snapshots",
     """
     List all the snapshots in the cluster in the same
     format as VolumeList.
@@ -609,7 +643,7 @@ Api.snapshotInfo = GET('SnapshotGetInfo/{snapshotName}', SnapshotName, returns=s
     Return general information about the distribution of the snapshot's data on the
     disks.
     """)
-Api.snapshotCreate = POST('VolumeSnapshot/{volumeName}', VolumeName, json=sp.VolumeSnapshotDesc, returns=ApiOkVolumeCreate, multiCluster=True).doc("Snapshot a volume",
+Api.snapshotCreate = POST('VolumeSnapshot/{volumeName}', VolumeName, json=sp.VolumeSnapshotDesc, returns=ApiOkSnapshotCreate, multiCluster=True).doc("Snapshot a volume",
     """
     Create a snapshot of the given volume; the snapshot becomes the parent of
     the volume.
@@ -646,10 +680,18 @@ Api.exportsList = GET('ExportsList', returns={'exports': [sp.Export]}).doc("List
     """
     """)
 
+Api.volumeExportsList = GET('VolumeExportsList', returns={'exports': [sp.Export]}).doc("List exported volumes",
+    """
+    """)
+
 Api.snapshotsRemoteList = GET('SnapshotsRemoteList', returns={'snapshots': [sp.RemoteSnapshot]}).doc("List the available remote snapshots",
     """
     """)
 
+Api.volumesRemoteList = GET('VolumesRemoteList', returns={'volumes': [sp.RemoteSnapshot]}).doc("List the available remote volumes",
+    """
+    """)
+
 Api.snapshotsRemoteUnexport = POST('SnapshotsRemoteUnexport', json=sp.SnapshotsRemoteUnexport).doc("Instruct the remote location that we will no longer use those snapshots",
     """
     """)
@@ -734,3 +776,27 @@ Api.iSCSIConfigChange = POST('iSCSIConfig', json=sp.iSCSIConfigChange).doc("Modi
 Api.spDocSection("Remote", "")
 
 Api.locationsList = GET("LocationsList", returns={'locations': [sp.RemoteLocation]}).doc("List the registered remote locations", "")
+Api.locationAdd = POST("LocationAdd", json=sp.RemoteLocationBase).doc("Register a new remote location", """ """)
+Api.locationRemove = POST("LocationRemove", json={'location': sp.RemoteLocationName}).doc("Remove a remote location", """ """)
+Api.locationUpdate = POST("LocationUpdate", json=sp.RemoteLocationUpdateDesc).doc("Update a remote location", """ """)
+Api.locationRename = POST("LocationRename", json=sp.RemoteLocationRenameDesc).doc("Rename a remote location", """ """)
+
+Api.clustersList = GET("ClustersList", returns={'clusters': [sp.RemoteCluster]}).doc("List the registered remote clusters", "")
+Api.clusterAdd = POST("ClusterAdd", json=sp.RemoteClusterAddDesc).doc("Register a new remote cluster", """ """)
+Api.clusterRemove = POST("ClusterRemove", json=sp.RemoteClusterRemoveDesc).doc("Remove a remote cluster", """ """)
+Api.clusterRename = POST("ClusterRename", json=sp.RemoteClusterRenameDesc).doc("Rename a remote cluster", """ """)
+
+Api.remoteBridgesList = GET("RemoteBridgesList", returns={'remoteBridges': [sp.RemoteBridge]}).doc("List the registered remote bridges", "")
+Api.remoteBridgeAdd = POST("RemoteBridgeAdd", json=sp.RemoteBridgeAddDesc).doc("Register a new remote bridge", """ """)
+Api.remoteBridgeRemove = POST("RemoteBridgeRemove", json=sp.RemoteBridgeRemoveDesc).doc("Deregister a remote bridge", """ """)
+
+Api.iSCSISessionsInfo = GET('iSCSISessionsInfo', json=maybe(sp.iSCSIControllersQuery), returns=sp.iSCSISessionsInfo).doc("query iSCSI controllers for active sessions", """ """)
+Api.iSCSInterfacesInfo = GET('iSCSInterfacesInfo', json=maybe(sp.iSCSIControllersQuery), returns=sp.iSCSIControllersIntefacesInfo).doc("query iSCSI controllers for interfaces state", """ """)
+
+Api.allPeersActiveRequests = GET('AllPeersActiveRequests', json=maybe(sp.AllPeersActiveRequestsQuery), returns=sp.AllPeersActiveRequests).doc("query all peers for their status and active requests", """ """)
+
+Api.maintenanceList = GET("MaintenanceList", returns=sp.MaintenanceNodesList).doc("List the nodes in maintenance", "")
+Api.maintenanceSet = POST("MaintenanceSet", json=sp.MaintenanceSetDesc).doc("Set node in maintenance", "")
+Api.maintenanceComplete = POST("MaintenanceComplete", json=sp.MaintenanceCompleteDesc).doc("complete node's maintenance.", "")
+
+Api.volumeRevert = POST("VolumeRevert/{volumeName}", VolumeName, json=sp.VolumeRevertDesc, multiCluster=True).doc("revert volume to a snapshot discarding all its current data", """ """)
