@@ -169,6 +169,8 @@ REMOTE_LOCATION_NAME_SIZE = 64
 REMOTE_LOCATION_NAME_REGEX = VOLUME_NAME_REGEX
 CLUSTER_NAME_REGEX = VOLUME_NAME_REGEX
 CLUSTER_NAME_SIZE = 64
+REMOTE_CLUSTER_NAME_REGEX = VOLUME_NAME_REGEX
+REMOTE_CLUSTER_NAME_SIZE = 64
 VOLUME_TAG_NAME_REGEX = r'^[A-Za-z0-9_\-.:]+$'
 VOLUME_TAG_VALUE_REGEX = r'^[A-Za-z0-9_\-.:]*$'
 
@@ -192,6 +194,8 @@ MAX_BRIDGE_ID = MAX_PEERS_PER_SUBTYPE - 1
 MAX_CLIENT_ID = MAX_PEERS_PER_SUBTYPE - 1
 MAX_MGMT_ID = MAX_PEERS_PER_SUBTYPE - 1
 
+NO_DISK_ID = 2**32 - 2
+
 GENERATION_NONE = longType(-1)
 
 # Simple type validators
@@ -215,6 +219,7 @@ BridgeId = intRange('BridgeId', 1, MAX_BRIDGE_ID)
 
 DiskId = intRange('DiskID', 0, MAX_DISK_ID)
 DiskDescription = regex('DiskDescritpion', DISK_DESC_REGEX)
+DiskIdOrNoDiskId = eitherOr(DiskId, NO_DISK_ID)
 
 SnapshotName = nameValidator("SnapshotName", SNAPSHOT_NAME_REGEX, VOLUME_NAME_SIZE, 'list', 'status')
 SnapshotNameOrGlobalId = nameValidator("SnapshotNameOrGlobalId", SNAPSHOT_NAME_OR_GLOBAL_ID_REGEX, VOLUME_NAME_SIZE, 'list', 'status')
@@ -238,11 +243,15 @@ AttachmentRights = oneOf('AttachmentRights', 'rw', 'ro')
 
 ObjectState = namedEnum("ObjectState", "OBJECT_UNDEF OBJECT_OK OBJECT_OUTDATED OBJECT_IN_RECOVERY OBJECT_WAITING_FOR_VERSION OBJECT_WAITING_FOR_DISK OBJECT_DATA_NOT_PRESENT OBJECT_DATA_LOST OBJECT_WAINING_FOR_CHAIN OBJECT_WAIT_IDLE".split(' '))
 
+DiskState = oneOf('DiskState', 'DISK_NONE', 'DISK_UNKNOWN', 'DISK_DATA_INITIALIZING', 'DISK_DATA_PENDING_INSERT', 'DISK_DATA', 'DISK_DATA_STOPPING', 'DISK_DATA_FLUSH_WBC', 'DISK_DATA_STOPPED', 'DISK_STOPPING', 'DISK_EJECTED', 'DISK_JOURNAL', 'DISK_JOURNAL_PENDING')
+
 RemoteLocationName = nameValidator("RemoteLocationName", REMOTE_LOCATION_NAME_REGEX, REMOTE_LOCATION_NAME_SIZE, 'list')
+RemoteClusterName = nameValidator("RemoteClusterName", REMOTE_CLUSTER_NAME_REGEX, REMOTE_CLUSTER_NAME_SIZE, 'list')
 ClusterName = nameValidator("ClusterName", CLUSTER_NAME_REGEX, CLUSTER_NAME_SIZE)
 GlobalVolumeId = regex('Global Volume Id', r'[a-z0-9]+\.[a-z0-9]+\.[a-z0-9]+$')
 LocationId = regex('Global Location Id', r'[a-z0-9]+$')
 ClusterId = regex('Global Location Id', r'[a-z0-9]+\.[a-z0-9]+$')
+SubClusterId = regex('Subcluster Id', r'[a-z0-9]+$')
 
 iSCSIId = intRange('iSCSIId', 0, 0x0fff)
 iSCSIName = r'^[a-z0-9\-.:]+$'
@@ -253,6 +262,7 @@ DematerializationStatus = oneOf("pending", "allObjectsUsed", "complete")
 DeleteBlocked = oneOf("not blocked", "pending", "rebasing", "flushing", "volume not found", "generation mismatch", "disk down", "multiple children", "peer down", "write not completed", "recovering from remote", "parent recovering from remote", "unknown")
 MultiClusterState = oneOf("owner", "ownerExported", "slaveCopy", "clusterLocal", "clusterLocalExported", "clusterLocalAutoReconcile", "clusterLocalAutoReconcileExported")
 
+
 # NETWORK
 @JsonObject(mac=MacAddr)
 class NetDesc(object):
@@ -486,14 +496,14 @@ class DiskDescUpdate(object):
 
 # ACTIVE REQUESTS
 @JsonObject(requestId=str, requestIdx=int, volume=either(VolumeNameOrGlobalId, SnapshotNameOrGlobalId), address=longType, size=int,
-    op=oneOf('RequestOp', "read", "write", "merge", "system", "entries flush", "#bad_state", "#bad_drOp"), state=internal(str), prevState=internal(str), drOp=internal(str), msecActive=int)
+    op=oneOf('RequestOp', "read", "write", "merge", "system", "entries flush", "#bad_state", "#bad_drOp", "idle", "error recovery", "transaction", "data recovery"), state=internal(str), prevState=internal(str), drOp=internal(str), msecActive=int)
 class ActiveRequestDesc(object):
     '''
     requestId: A unique request ID that may be matched between clients and disks.
     requestIdx: A temporary local request identifier for this request on this client or disk.
     address: The offset in bytes within the logical volume.
     size: The size of the request in bytes.
-    op: The type of the requested operation; one of read, write, system, merge, entries flush, #bad_state, #bad_drOp
+    op: The type of the requested operation; one of read, write, system, merge, entries flush, #bad_state, #bad_drOp, idle, error recovery, transaction, data recovery
     state: An internal attribute used only for debugging. We strongly recommend that you do not use this attribute in any kind of automation.
     prevState: An internal attribute used only for debugging. We strongly recommend that you do not use this attribute in any kind of automation.
     drOp: An internal attribute used only for debugging. We strongly recommend that you do not use this attribute in any kind of automation.
@@ -549,8 +559,8 @@ class VolumeLimits(object):
     '''
 
 
-@JsonObject(id=internal(longType), parentName=eitherOr(SnapshotName, ""), templateName=eitherOr(VolumeTemplateName, ""),
-    size=VolumeSize, replication=VolumeReplication,
+@JsonObject(id=internal(longType), parentName=eitherOr(SnapshotNameOrGlobalId, ""), templateName=eitherOr(VolumeTemplateName, ""),
+    size=VolumeSize, replication=VolumeReplication, globalId=GlobalVolumeId,
     placeAll=PlacementGroupName, placeTail=PlacementGroupName, placeHead=PlacementGroupName,
     parentVolumeId=internal(longType), originalParentVolumeId=internal(longType), visibleVolumeId=longType, templateId=internal(longType),
     objectsCount=int, creationTimestamp=longType, flags=internal(int), tags=maybe({VolumeTagName: VolumeTagValue}),
@@ -563,6 +573,7 @@ class VolumeSummaryBase(VolumeLimits):
     templateName: The template that the volume/snapshot's settings are taken from.
     size: The volume/snapshots's size in bytes.
     replication: The number of copies/replicas kept.
+    globalId: The global identifier.
     placeAll: The name of a placement group which describes the disks to be used for all but the last replica.
     placeTail: The name of a placement group which describes the disks to be used for the last replica, the one used for reading.
     placeHead: The name of a placement group which describes the disks to be used for the first replica.
@@ -584,8 +595,8 @@ class VolumeSummary(VolumeSummaryBase):
     '''
 
 
-@JsonObject(name=SnapshotName, onVolume=VolumeNameOrGlobalId,
-    autoName=bool, bound=bool, deleted=bool, transient=bool, targetDeleteDate=maybe(int), globalId=GlobalVolumeId,
+@JsonObject(name=SnapshotNameOrGlobalId, onVolume=VolumeNameOrGlobalId,
+    autoName=bool, bound=bool, deleted=bool, transient=bool, targetDeleteDate=maybe(int),
     recoveringFromRemote=bool,
     backup=maybe(bool), backupOfVisibleVolumeId=maybe(longType), backupOfGlobalId=maybe(GlobalVolumeId),
     createdFromVisibleVolumeId=maybe(internal(longType)),
@@ -604,7 +615,6 @@ class SnapshotSummary(VolumeSummaryBase):
     deleted: Is this snapshot currently being deleted.
     transient: Is this a transient snapshot. Transient snapshots are internally created when cloning a volume. They cannot be attached as they may be garbage-collected at any time.
     targetDeleteDate: Scheduled date for the snapshot to be deleted. Unix timestamp
-    globalId: The global snapshot identifier.
     recoveringFromRemote: Is this snapshot's data currently being transferred from a remote location.
     backup: Is this snapshot a backup of a remote snapshot.
     backupOfVisibleVolumeId: The numeric global ID of the snapshot that has been backed up to this snapshot.
@@ -747,7 +757,7 @@ class SnapshotUpdateDesc(VolumePolicyDesc):
     '''
 
 
-@JsonObject(parentName=maybe(SnapshotName))
+@JsonObject(parentName=maybe(SnapshotNameOrGlobalId))
 class VolumeRebaseDesc(object):
     '''
     parentName: The name of one of the volume's parents on which to re-base. If left out, it will be re-based to base.
@@ -787,7 +797,7 @@ class VolumeReassignDesc(object):
     '''
 
 
-@JsonObject(snapshot=SnapshotName, detach=maybe(DetachClientsList), ro=maybe([ClientId]), force=False)
+@JsonObject(snapshot=SnapshotNameOrGlobalId, detach=maybe(DetachClientsList), ro=maybe([ClientId]), force=False)
 class SnapshotReassignDesc(object):
     '''
     snapshot: The name of the snapshot which should be reassigned.
@@ -1033,12 +1043,30 @@ class SnapshotFromRemoteDesc(object):
     placeAll: The name of a placement group which describes the disks to be used for all but the last replica.
     placeTail: The name of a placement group which describes the disks to be used for the last replica, the one used for reading.
     placeHead: The name of a placement group which describes the disks to be used for the first replica.
+    replication: The number of copies/replicas kept.
+    template: The name of the template that the settings of the new volume are based on.
     export: Auto-export the snapshot after creating it. e.g. for backup.
     tags: Arbitrary short name/value pairs stored with the snapshot.
     '''
 
 
-@JsonObject(snapshot=SnapshotName, location=RemoteLocationName)
+@JsonObject(remoteLocation=RemoteLocationName, remoteId=GlobalVolumeId, name=VolumeName, placeAll=maybe(PlacementGroupName), placeTail=maybe(PlacementGroupName), placeHead=maybe(PlacementGroupName), replication=maybe(VolumeReplication), template=maybe(VolumeTemplateName), export=maybe(bool), tags=maybe({VolumeTagName: VolumeTagValue}))
+class VolumeFromRemoteDesc(object):
+    '''
+    remoteLocation: The name of the remote location to fetch the snapshot from.
+    remoteId: The global snapshot identifier.
+    name: The name of the new volume.
+    placeAll: The name of a placement group which describes the disks to be used for all but the last replica.
+    placeTail: The name of a placement group which describes the disks to be used for the last replica, the one used for reading.
+    placeHead: The name of a placement group which describes the disks to be used for the first replica.
+    replication: The number of copies/replicas kept.
+    template: The name of the template that the settings of the new volume are based on.
+    export: Auto-export the volume after creating it. e.g. for backup.
+    tags: Arbitrary short name/value pairs stored with the volume.
+    '''
+
+
+@JsonObject(snapshot=SnapshotNameOrGlobalId, location=RemoteLocationName)
 class SnapshotExportDesc(object):
     '''
     snapshot: The name of the snapshot.
@@ -1046,7 +1074,7 @@ class SnapshotExportDesc(object):
     '''
 
 
-@JsonObject(snapshot=SnapshotName, location=maybe(RemoteLocationName), all=maybe(bool), force=maybe(bool))
+@JsonObject(snapshot=SnapshotNameOrGlobalId, location=maybe(RemoteLocationName), all=maybe(bool), force=maybe(bool))
 class SnapshotUnexportDesc(object):
     '''
     snapshot: The name of the snapshot.
@@ -1065,10 +1093,11 @@ class VolumeBackupDesc(object):
     '''
 
 
-@JsonObject(remoteId=GlobalVolumeId)
+@JsonObject(remoteId=GlobalVolumeId, snapshotGlobalId=GlobalVolumeId)
 class VolumesGroupBackupSingle(object):
     '''
     remoteId: the globally unique id of the backup.
+    snapshotGlobalId: the globally unique id of the backup.
     '''
 
 
@@ -1105,7 +1134,18 @@ class VolumeAcquireDesc(object):
     '''
 
 
-@JsonObject(name=VolumeNameOrGlobalId, location=RemoteLocationName, creationTimestamp=longType, size=VolumeSize, remoteId=GlobalVolumeId, onVolume=maybe(VolumeNameOrGlobalId), localSnapshot=maybe(SnapshotNameOrGlobalId), tags=maybe({VolumeTagName: VolumeTagValue}))
+@JsonObject(
+    name=VolumeNameOrGlobalId,
+    location=RemoteLocationName,
+    creationTimestamp=longType,
+    size=VolumeSize,
+    remoteId=GlobalVolumeId,
+    onVolume=maybe(VolumeNameOrGlobalId),
+    localSnapshot=maybe(SnapshotNameOrGlobalId),
+    tags=maybe({VolumeTagName: VolumeTagValue}),
+    createdFromGlobalId=internal(GlobalVolumeId),
+    createdFromVisibleVolumeId=internal(longType)
+)
 class RemoteSnapshot(object):
     '''
     name: The name of the snapshot.
@@ -1120,14 +1160,142 @@ class RemoteSnapshot(object):
 
 
 @JsonObject(id=LocationId, name=RemoteLocationName)
-class RemoteLocation(object):
+class RemoteLocationBase(object):
     '''
     id: A StorPool-provided unique location id.
     name: The human-readable location name.
     '''
 
 
-@JsonObject(snapshot=SnapshotName, location=RemoteLocationName, globalId=GlobalVolumeId, backingUp=maybe(bool), volumeId=internal(longType), visibleVolumeId=internal(longType))
+@JsonObject(sendBufferSize=longType, recvBufferSize=longType)
+class RemoteLocation(RemoteLocationBase):
+    '''
+    sendBufferSize: the size of the TCP send buffer for the location
+    recvBufferSize: the size of the TCP recieve buffer for the location
+    '''
+
+
+@JsonObject(
+    location=RemoteLocationName,
+    sendBufferSize=maybe(longType),
+    recvBufferSize=maybe(longType)
+)
+class RemoteLocationUpdateDesc(object):
+    '''
+    location: The human-readable location name.
+    sendBufferSize: the size of the TCP send buffer for the location
+    recvBufferSize: the size of the TCP recieve buffer for the location
+    '''
+
+
+@JsonObject(location=RemoteLocationName, name=RemoteLocationName)
+class RemoteLocationRenameDesc(object):
+    '''
+    location: The human-readable location name.
+    name: The new human-readable location name.
+    '''
+
+
+@JsonObject(id=SubClusterId, location=RemoteLocationName)
+class RemoteClusterBase(object):
+    '''
+    id: A StorPool-provided unique cluster subid
+    location: the cluster's location name
+    '''
+
+
+@JsonObject(name=maybe(RemoteClusterName))
+class RemoteClusterAddDesc(RemoteClusterBase):
+    '''
+    name: The human-readable cluster name
+    '''
+
+
+@JsonObject(name=RemoteClusterName)
+class RemoteCluster(RemoteClusterBase):
+    '''
+    name: The human-readable cluster name
+    '''
+
+
+@JsonObject(
+    name=maybe(RemoteClusterName),
+    location=maybe(RemoteLocationName),
+    id=maybe(SubClusterId),
+    clusterWillNotBeComingBack=maybe(bool)
+)
+class RemoteClusterRemoveDesc(object):
+    '''
+    name: The human-readable cluster name.
+    location: the cluster's location name
+    id: A StorPool-provided unique cluster subid
+    clusterWillNotBeComingBack: Set this flag if the remote cluster will not be registered again in the future. Default is false.
+    '''
+
+
+@JsonObject(cluster=RemoteClusterName, name=RemoteClusterName)
+class RemoteClusterRenameDesc(object):
+    '''
+    cluster: The human-readable cluster name.
+    name: The new human-readable cluster name.
+    '''
+
+
+@JsonObject(
+    ip=str,
+    publicKey=str,
+    minimumDeleteDelay=maybe(longType),
+    noCrypto=maybe(bool)
+)
+class RemoteBridgeBase(object):
+    '''
+    ip: the ip address of the remote bridge
+    publicKey: the public key of the remote bridge
+    minimumDeleteDelay: minimum value for the deferred deletion
+    noCrypto: Set this flag if the connection with the remote bridge should not be encrypted
+    '''
+
+
+@JsonObject(location=RemoteLocationName)
+class RemoteBridgeAddLocationDesc(RemoteBridgeBase):
+    '''
+    location: the location of the remote bridge
+    '''
+
+
+@JsonObject(remote=RemoteClusterName)
+class RemoteBridgeAddClusterDesc(RemoteBridgeBase):
+    '''
+    remote: the cluster of the remote bridge
+    '''
+
+
+RemoteBridgeAddDesc = either(
+    RemoteBridgeAddLocationDesc, RemoteBridgeAddClusterDesc
+)
+
+
+@JsonObject(ip=str)
+class RemoteBridgeRemoveDesc(object):
+    '''
+    ip: the ip address of the remote bridge
+    '''
+
+
+@JsonObject(
+    location=RemoteLocationName,
+    cluster=maybe(RemoteClusterName),
+    remote=either(RemoteClusterName, RemoteLocationName)
+)
+class RemoteBridge(RemoteBridgeBase):
+    '''
+    location: the location of the remote bridge
+    cluster: the cluster of the remote bridge
+    remote: the cluster or the location of the remote bridge
+    '''
+
+
+@JsonObject(snapshot=SnapshotNameOrGlobalId, location=RemoteLocationName, globalId=GlobalVolumeId, backingUp=maybe(bool), volumeId=internal(longType), visibleVolumeId=internal(longType))
 class Export(object):
     '''
     snapshot: The name of the snapshot.
@@ -1154,11 +1322,16 @@ class SnapshotsRemoteUnexport(object):
     '''
 
 
-@JsonObject(volume=VolumeNameOrGlobalId, name=maybe(SnapshotName))
+@JsonObject(
+    volume=VolumeNameOrGlobalId,
+    name=maybe(SnapshotName),
+    tags=maybe({VolumeTagName: VolumeTagValue})
+)
 class GroupSnapshotSpec(object):
     '''
     volume: The name of the volume to create a snapshot of.
     name: The name of the snapshot to create.
+    tags: Arbitrary short name/value pairs to be stored with the snapshot.
     '''
 
 
@@ -1169,19 +1342,21 @@ class GroupSnapshotsSpec(object):
     '''
 
 
-@JsonObject(volume=VolumeNameOrGlobalId, snapshot=maybe(SnapshotName), remoteId=GlobalVolumeId)
+@JsonObject(volume=VolumeNameOrGlobalId, volumeGlobalId=GlobalVolumeId, snapshot=maybe(SnapshotNameOrGlobalId), remoteId=GlobalVolumeId, snapshotGlobalId=GlobalVolumeId)
 class GroupSnapshotResult(object):
     '''
     volume: The name of the source volume.
+    volumeGlobalId: The globally unique id of the volume.
     snapshot: The name of the created snapshot.
     remoteId: The globally unique id of the created snapshot.
+    snapshotGlobalId: The globally unique id of the created snapshot.
     '''
 
 
-@JsonObject(volumes=[GroupSnapshotResult])
+@JsonObject(snapshots=[GroupSnapshotResult])
 class GroupSnapshotsResult(object):
     '''
-    volumes: The volumes to create snapshots of.
+    snapshots: The volumes to create snapshots of.
     '''
 
 
@@ -1396,3 +1571,226 @@ class iSCSIConfigChange(object):
     '''
     commands: The actual iSCSI configuration commands.
     '''
+
+
+@JsonObject(nopOut=int, scsi=int, task=int, login=int, text=int, dataOut=int, logout=int, snack=int, nopIn=int, scsiRsp=int, taskRsp=int, loginRsp=int, textRsp=int, dataIn=int, logoutRsp=int, r2t=int, reject=int)
+class iSCSISessionStats(object):
+    '''
+    '''
+
+
+@JsonObject(inFreeList=int, dataOut=int, queued=int, processing=int, dataResp=int, aborted=int)
+class iSCSISessionTasks(object):
+    '''
+    '''
+
+
+@JsonObject(totalPacketsIn=int, totalBytesIn=int, newPacketsIn=int, newBytesIn=int, discardedPackets=int, discardedBytes=int, dataHoles=int, totalPacketsOut=int,
+    totalBytesOut=int, newPacketsOut=int, newBytesOut=int, retransmittedPackets=int, retransmittedBytes=int, retransmitsTimeout=int, retransmitsAcks=int, retransmitsAcks2=int)
+class iSCSISessionTcpStats(object):
+    '''
+    '''
+
+
+@JsonObject(portalIP=str, portalPort=int, initiatorIP=str, initiatorPort=int, wscale=int, localMSS=int, remoteMSS=int, remoteWindowSize=int, hwPort=int, stats=iSCSISessionTcpStats)
+class iSCSISessionTcp(object):
+    '''
+    '''
+
+
+@JsonObject(status=str, controllerId=iSCSIId, connectionId=maybe(int), ISID=maybe(str), timeCreated=maybe(int), target=maybe(str), initiator=maybe(str),
+    stats=maybe(iSCSISessionStats), tasks=maybe(iSCSISessionTasks), tcp=maybe(iSCSISessionTcp))
+class iSCSISessionInfo(object):
+    '''
+    satus: ok or other
+    controllerId: controller id
+    ISID: session ISID
+    timeCreated: unix timestamp when session was established
+    targset: iSCSI name of the target
+    initiator: iSCSI name of the initiator
+    stats: session stats
+    tasks: currently active tasks count by state
+    tcp: session TCP info
+    '''
+
+
+@JsonObject(sessions=[iSCSISessionInfo])
+class iSCSISessionsInfo(object):
+    '''
+    sessions: list of sessions
+    '''
+
+
+@JsonObject(port=int, state=str, interfaceName=str, MAC=str, resolveInterfaceName=str)
+class iSCSIControllerInterfaceInfo(object):
+    '''
+    '''
+
+
+@JsonObject(msecsTimeout=maybe(int), controllerIds=maybe([iSCSIId]))
+class iSCSIControllersQuery(object):
+    '''
+    msecsTimeout: 0 means no timeout
+    controllerIds: ask only those controllers
+    '''
+
+
+@JsonObject(status=str, controllerId=iSCSIId, interfaces=maybe([iSCSIControllerInterfaceInfo]))
+class iSCSIControllerIntefacesInfo(object):
+    '''
+    '''
+
+
+@JsonObject(controllers=[iSCSIControllerIntefacesInfo])
+class iSCSIControllersIntefacesInfo(object):
+    '''
+    '''
+
+
+# ALL PEERS ACTIVE REQUESTS
+@JsonObject(type=oneOf('Id', 'bridge', 'server', 'iSCSI', 'client', 'unknown'), id=str)
+class AllPeersActiveRequestsServiceDesc(object):
+    '''
+    type: type of the service
+    id: identificator for the service of the given type
+    '''
+
+
+@JsonObject(peerId=PeerId, service=AllPeersActiveRequestsServiceDesc)
+class AllPeersActiveRequestsPeerDesc():
+    '''
+    peerId: peer id
+    service: identification json for the service
+    '''
+
+
+@JsonObject(status=oneOf('Status', 'too_old', 'peer_down', 'peerDone', 'timeout', 'streamNotConnected', 'streamOverfilled', 'invalidResponse'))
+class AllPeersActiveRequestsSimpleStats(AllPeersActiveRequestsPeerDesc):
+    '''
+    status: status of the request to the service
+    '''
+
+
+@JsonObject(status=const('diskExpected'), diskId=DiskId, lastState=oneOf('LastState', 'up', 'down'))
+class AllPeersActiveRequestsDiskExpected(AllPeersActiveRequestsPeerDesc):
+    '''
+    status: "diskExpected"
+    diskId: disk id
+    lastState: last known state of the disk
+    '''
+
+
+@JsonObject(status=const('diskState'), diskId=DiskId, diskState=DiskState, objectsWaitingForVersion=internal(int), objectsOutdated=internal(int), objectsOutdatedRemote=internal(int))
+class AllPeersActiveRequestsDiskStatus(AllPeersActiveRequestsPeerDesc):
+    '''
+    status: "diskState"
+    diskId: disk id
+    distState: state of the disk
+    objectsWaitingForVersion: An internal attribute used only for debugging. We strongly recommend that you do not use this attribute in any kind of automation.
+    objectsOutdated: An internal attribute used only for debugging. We strongly recommend that you do not use this attribute in any kind of automation.
+    objectsOutdatedRemote: An internal attribute used only for debugging. We strongly recommend that you do not use this attribute in any kind of automation.
+    '''
+
+
+@JsonObject(diskId=DiskIdOrNoDiskId, peerId=PeerId, service=maybe(AllPeersActiveRequestsServiceDesc))
+class AllPeersActiveRequestsRequestPeer(object):
+    '''
+    diskId: disk id
+    peerId: peer id
+    service: identification json for the service
+    '''
+
+
+@JsonObject(
+    status=const('request'),
+    diskId=DiskIdOrNoDiskId,
+    requestIdx=int,
+    requestId=str,
+    volumeId=longType,
+    volume=maybe(either(VolumeNameOrGlobalId, SnapshotNameOrGlobalId)),
+    address=longType,
+    size=int,
+    op=oneOf('RequestOp', "read", "write", "merge", "system", "entries flush", "#bad_state", "#bad_drOp", "idle", "error recovery", "transaction", "data recovery"),
+    usecActive=int,
+    peers=[AllPeersActiveRequestsRequestPeer],
+    state=internal(str),
+    prevState=internal(str),
+    drOp=internal(maybe(str)),
+)
+class AllPeersActiveRequestsRequest(AllPeersActiveRequestsPeerDesc):
+    '''
+    status: "request"
+    requestId: A unique request ID that may be matched between clients and disks.
+    requestIdx: A temporary local request identifier for this request on this client or disk.
+    volumeId: id of the volume associated with the request
+    volume: global id or name of the volume associated with the request
+    address: The offset in bytes within the logical volume.
+    size: The size of the request in bytes.
+    op: The type of the requested operation; one of read, write, system, merge, entries flush, #bad_state, #bad_drOp, idle, error recovery, transaction, data recovery
+    usecActive: Time in microseconds since the request was submitted.
+    peers: list of peers associated with the request
+    state: An internal attribute used only for debugging. We strongly recommend that you do not use this attribute in any kind of automation.
+    prevState: An internal attribute used only for debugging. We strongly recommend that you do not use this attribute in any kind of automation.
+    drOp: An internal attribute used only for debugging. We strongly recommend that you do not use this attribute in any kind of automation.
+    '''
+
+
+@JsonObject(requests=[either(AllPeersActiveRequestsDiskExpected, AllPeersActiveRequestsDiskStatus, AllPeersActiveRequestsRequest, AllPeersActiveRequestsSimpleStats)])
+class AllPeersActiveRequests(object):
+    '''
+    requests: list containing service status messages and active requests descriptions
+    '''
+
+
+@JsonObject(msecsTimeout=maybe(int))
+class AllPeersActiveRequestsQuery(object):
+    '''
+    msecsTimeout: 0 is no timeout
+    '''
+
+
+# MAINTENANCE MODE
+@JsonObject(nodeId=int, started=int, duration=int, description=str, overrun=bool, remaining=maybe(int))
+class MaintenanceNodeDesc(object):
+    '''
+    nodeId: The id of the node in maintenance,
+    started: unix timestamp the maintenance was started
+    duration: planned duration of the maintenance
+    description: human-readable information
+    overrun: true if the duration has passed
+    rmaining: seconds until the end of the duration
+    '''
+
+
+@JsonObject(nodes=[MaintenanceNodeDesc])
+class MaintenanceNodesList(object):
+    '''
+    A detailed listing of all nodes in maintenance and the services running on them.
+    '''
+
+
+@JsonObject(nodeId=int, duration=int, description=str, minOkReplicas=maybe(int), maxReplicationDecrease=maybe(int), votingSafetyMargin=maybe(int), maxNodesInMaintenance=maybe(int))
+class MaintenanceSetDesc(object):
+    '''
+    nodeId: The id of the node which will be set in maintenance.
+    duration: Period of time after which the maintenance will expire automatically.
+    description: human-readable information
+    minOkReplicas: the minimum ok copies of all data that has a copy on the node that must remain while the node is down
+    maxReplicationDecrease: same as minOkReplicas = volume.replication - maxReplicationDecrease, but for each volume
+    votingSafetyMargin: number of voting nodes that must be alive above the 1/2 + 1 minimum
+    maxNodesInMaintenance: maximum number of nodes simultaneously in maintenance
+    '''
+
+
+@JsonObject(nodeId=int)
+class MaintenanceCompleteDesc(object):
+    '''
+    nodeId: The id of the node to unset maintenance mode for
+    '''
+
+
+@JsonObject(toSnapshot=SnapshotNameOrGlobalId)
+class VolumeRevertDesc(object):
+    '''
+    toSnapshot: name of the snapshot to revert to
+    '''
